#include "specialmath.h"

void scaleVector(double alpha,double *v,int n) {
   int i;
   for (i=0;i<n;i++) v[i]*=alpha;
}

double vectorSum1(const double *V,int n) {
   double sum=0;
   int i;
   for (i=0;i<n;i++) {
      sum+=V[i];
   }
   return sum;
}

double vectorSum2(const double *V,int n) {
   double sum=0;
   int i;
   for (i=0;i<n;i++) {
      double help=V[i];
      sum+=help*help;
   }
   return sum;
}

void calcSquareSums(double *sums,const double *A,int m,int n) {
   int i;
   for (i=0;i<n;i++) sums[i]=vectorSum2(A+m*i,m);
}

void storeAlphaUtoV(double alpha,const double *U,double *V,int n) {
   while (n>=4) {
      V[0]=alpha*U[0];
      V[1]=alpha*U[1];
      V[2]=alpha*U[2];
      V[3]=alpha*U[3];
      V+=4; U+=4; n-=4;
   }
   switch(n) { 
      case 3 :
         V[2]=alpha*U[2];
      case 2 :
         V[1]=alpha*U[1];
      case 1 : 
         V[0]=alpha*U[0];
   }
}


void addAlphaUtoV(double alpha,const double *U,double *V,int n) {
   while (n>=4) {
      V[0]+=alpha*U[0];
      V[1]+=alpha*U[1];
      V[2]+=alpha*U[2];
      V[3]+=alpha*U[3];
      V+=4; U+=4; n-=4;
   }
   switch(n) { 
      case 3 :
         V[2]+=alpha*U[2];
      case 2 :
         V[1]+=alpha*U[1];
      case 1 : 
         V[0]+=alpha*U[0];
   }
}


double sqDist(const double *a,const double *b,int n) {
	int i;
	double d=0.0;
   for (i=0;i<n;i++) {
      double dif=a[i]-b[i];
      d+=dif*dif;
   }
   return d;
}

void calcSqDistances(double *d,const double *X,const double *x,int q,int N) {
int i,j;
   for (i=0;i<N;i++) {
      double t=0.0;
      for (j=0;j<q;j++) {
         double t2=X[j]-x[j];
         t+=t2*t2;
      }
      X+=q;
      
      d[i]=t;
   }
} 

void shiftXcalcDists(double *d,double *Xs,const double *X,const double *x,int q,int N) {
   int i,m;
   
   for (i=0;i<N;++i) {
      double tmp=0.0;
      for (m=0;m<q;++m) {
         double diff;
         Xs[m+i*q]=diff=X[m+i*q] - x[m];
         tmp+=diff*diff;
      }
      d[i]=tmp;
   }
}
    
double dotProduct(const double *U,const double *V,int n) {
   double t=0.0;
   while (n>=4) {
      t+=V[0]*U[0];
      t+=V[1]*U[1];
      t+=V[2]*U[2];
      t+=V[3]*U[3];
      V+=4; U+=4; n-=4;
   }
   switch(n) {
      case 3 : t+=V[2]*U[2];
      case 2 : t+=V[1]*U[1];
      case 1 : t+=V[0]*U[0];
   }
   return t;
}


void addSparseToFull(const int *table,const double *s,double *f,int n) {
   int i;
   for (i=0;i<n;i++) {
      f[table[i]]+=s[i];
   }
}

void multMatrixVector(double *y,const double *A,const double *x,int m,int n) {
   int i;
   storeAlphaUtoV(x[0],A,y,m);
   for (i=1;i<n;i++) {
      addAlphaUtoV(x[i],A+m*i,y,m);
   }
}

void multMatrixVectorSp(double *y,const double *A,const double *x,int m,int nz, const int *ir) {
   int i;
   
   if (nz==0) {
      for (i=0;i<m;i++) y[i]=0.0;
      return;
   }
   storeAlphaUtoV(x[0],A+m*ir[0],y,m);
   for (i=1;i<nz;i++) {
      addAlphaUtoV(x[i],A+m*ir[i],y,m);
   }
}

void multMatrixTVector(double *y,const double *A,const double *x,int m,int n) {
   int i;
   for (i=0;i<n;i++) y[i]=dotProduct(A+i*m,x,m);
}

void multDiagMatrix(double *As, const double *s,const double *A,int q,int N) {
   int m,n;
   for (n=0;n<N;n++) {
      for (m=0;m<q;m++) {
         As[m]=s[m]*A[m];
      }
      As+=m;
      A+=m;
   }
}


/* translated from FORTRAN syminv */

/*  IN SITU INVERSION OF A REAL SYMMETRIC MATRIX BY L.D.L(T) */
/*  DECOMPOSITION. numNeg = NUMBER OF NEGATIVE EIGENVALUES. */
/*  ROUTINE FAILS (IFAIL = N+1) IF THE DECOMPOSITION IS UNDEFINED. */
/*  J.H.WILKINSON AND C.REINSCH, LINEAR ALGEBRA, I/1 */
/*  FOR VERSION 12 - NO LONGER NECESSARY TO RESTORE UPPER TRIANGLE */

int invertSymmetricMatrix(double *A, int n) {

   /* Local variables */
   int i, j, k;
   double x, y, z;
   int i1, j1, n1, numNeg;
   
   int lda=n;


   /* Function Body */
   numNeg = 0;
   x = A[0];
   if (x < 0.) ++numNeg;

   if (x == 0.) return -1;
    
   A[0] = 1. / x;
   if (n==1) return numNeg;
   /*   A-=n+1; */

   /*  FORM L */

   for (i = 1; i <= n-1; ++i) {
      i1 = i - 1;

      if (i1 >= 1) {
         for (j = 1; j <= i1; ++j) {
            j1 = j - 1;
            x = A[i + j * lda];
            for (k = 0; k <= j1; ++k) {
               x -= A[i + k * lda] * A[j + k * lda];
            }
            A[i + j * lda] = x;
         }
      }

      x = A[i + i * lda];

      for (k = 0; k <= i1; ++k) {
         y = A[i + k * lda];
         z = y * A[k + k * lda];
         A[i + k * lda] = z;
         x -= y * z;
      }
      if (x < 0.) {
         ++numNeg;
      }
      if (x == 0.) return -1;
      A[i + i * lda] = 1. / x;
   }
   /*  INVERT L */
   n1 = n - 2;

   for (i = 0; i <= n1; ++i) {
      i1 = i + 1;
      A[i1 + i * lda] = -A[i1 + i * lda];
      if (i1 > n1) continue;
      for (j = i1; j <= n1; ++j) {
         j1 = j + 1;
         x = -A[j1 + i * lda];
         for (k = i1; k <= j; ++k) {
            x -= A[j1 + k * lda] * A[k + i * lda];
         }
         A[j1 + i * lda] = x;
      }
   }
   /*  FORM INVERSE OF A */

   for (i = 0; i <= n1; ++i) {
      i1 = i + 1;
      x = A[i + i * lda];
      for (k = i1; k <= n-1; ++k) {
         y = A[k + i * lda];
         z = y * A[k + k * lda];
         A[k + i * lda] = z;
         x += y * z;
      }
      A[i + i * lda] = x;
      if (i1 > n1) continue;
      for (j = i1; j <= n1; ++j) {
         j1 = j + 1;
         x = A[j + i * lda];
         for (k = j1; k <= n-1; ++k) {
            x += A[k + j * lda] * A[k + i * lda];
         }
         A[j + i * lda] = x;
      }
   }
   /*  COPY INVERSE INTO UPPER TRIANGLE    ----- */
   for (j=1;j<n;j++) {
      for (i=0;i<j;i++) {
         A[i+j*lda] = A[j+i*lda];
      }
   }
   return numNeg;
} 

