#include "ukr_matlab.h"
#include "specialmath.h"

void mexFunction(int nlhs, mxArray *plhs[], int nrhs,const mxArray *prhs[]) {
   UKR_Model model;
   const double *x;
   double *b;
   double *dens=NULL;
   UKR_Workspace W;
   int N;
   
   if (nrhs<2) mexErrMsgTxt("Not enough arguments.");
   if (nrhs>2) mexErrMsgTxt("Too many arguments.");
   
   model = ukr_model_from_matlab(prhs[1]);

   if (model.X==NULL) mexErrMsgTxt("Model does not contain 'X'");
   
   x = mxGetPr(prhs[0]);
   if (mxGetM(prhs[0])!=model.q) mexErrMsgTxt("Dimension of X does not match model.");
   N=mxGetN(prhs[0]);
     
   ukr_aux_alloc_workspace(model,&W);
      
   if (nlhs>1) {
      plhs[1]=mxCreateDoubleMatrix(1,N,mxREAL);
      dens=mxGetPr(plhs[1]);
   }
   
   if (model.kernel > UKR_KERNEL_SPARSE) {
   	int i,j;
      int *ir,*jc;
      
      plhs[0]=mxCreateSparse(model.N,N,model.N*N,mxREAL);
      b=mxGetPr(plhs[0]);
      ir=mxGetIr(plhs[0]);
      jc=mxGetJc(plhs[0]);
      jc[0]=0;
      
      for (j=0;j<N;j++) {
         shiftXcalcDists(W.K,W.Xx,model.X,x,model.q,model.N);
         ukr_kp_dists(W.K,&W,model,0,NULL);
         if (W.nz<model.N) ukr_aux_shiftX(W.Xx, model.q, W.nz, W.ir);         
         if (W.nz>0 && ukr_aux_b(&W,model)) {
            for (i=0;i<W.nz;i++) {
               *b++=W.B[i];
               *ir++=W.ir[i];
            }
            jc[j+1]=jc[j]+W.nz;
         } else {
            jc[j+1]=jc[j];   
         }
         x+=model.q;
         if (dens) dens[j]=W.sumK/model.N;
      }
   } else {
    	int i,j;
      
      plhs[0]=mxCreateDoubleMatrix(model.N,N,mxREAL);
      b=mxGetPr(plhs[0]);
      
      for (j=0;j<N;j++) {
         shiftXcalcDists(W.K,W.Xx,model.X,x,model.q,model.N);
         ukr_kp_dists(W.K,&W,model,0,NULL);
         if (W.nz<model.N) ukr_aux_shiftX(W.Xx, model.q, W.nz, W.ir);         
         if (!ukr_aux_b(&W,model)) {
            for (i=0;i<model.N;i++) b[i]=0;
         } else {
            for (i=0;i<model.N;i++) {
               b[i]=W.B[i];
            }
         }
         b+=model.N;
         x+=model.q;
         if (dens) dens[j]=W.sumK/model.N;         
      }      
   } 
   
   ukr_aux_free_workspace(W);
}
