#include "ukr_matlab.h"
#include "specialmath.h"


void mexFunction(int nlhs, mxArray *plhs[], int nrhs,const mxArray *prhs[]) {
   UKR_Model model;
   UKR_Workspace W;
   const double *x;
   double *b,*J;
   int i;
   
   if (nrhs<2) mexErrMsgTxt("Not enough arguments.");
   if (nrhs>2) mexErrMsgTxt("Too many arguments.");
   
   model = ukr_model_from_matlab(prhs[1]);

   if (model.X==NULL) mexErrMsgTxt("Model does not contain 'X'");
   
   x = ukr_mx_get_matrix(prhs[0], model.q, 1);
   if (x==NULL) mexErrMsgTxt("Parameter x must be a 'q x 1' vector.");
   
   if (!ukr_aux_alloc_workspace(model,&W)) mexErrMsgTxt("Sorry, out of memory.");
   
   plhs[0]=mxCreateDoubleMatrix(model.N,1,mxREAL);
   b=mxGetPr(plhs[0]);   
   plhs[1]=mxCreateDoubleMatrix(model.N,model.q,mxREAL);
   J=mxGetPr(plhs[1]);
      
      
   shiftXcalcDists(W.K,W.Xx,model.X,x,model.q,model.N);
   ukr_kp_dists(W.K,&W,model,0,NULL);
   if (W.nz<model.N) ukr_aux_shiftX(W.Xx, model.q, W.nz, W.ir);
   ukr_aux_bJ(J,&W,model);
   
   for (i=0;i<W.nz;i++) b[W.ir[i]]=W.B[i];
   if (nlhs>2) {
      plhs[2]=mxCreateDoubleScalar(W.sumK/model.N);
   }
   ukr_aux_free_workspace(W);
	
}
