#include "ukr_matlab.h"

const char FS_ERR_MSG[]="Model does not contain 'Y': Pass a correct structure as a 3rd parameter.";
const char DS_ERR_MSG[]="Parameter 'y' does not match model.";

void mexFunction(int nlhs, mxArray *plhs[], int nrhs,const mxArray *prhs[]) {
   UKR_Model model;
   UKR_G_Struct gs;
   UKR_Workspace W;
   
   const double *x;
   double err;
   double *g,*H;

   if (nrhs<3) mexErrMsgTxt("Not enough arguments.");
   if (nrhs>3) mexErrMsgTxt("Too many arguments.");
   
   model = ukr_model_from_matlab(prhs[1]);

   if (model.X==NULL) mexErrMsgTxt("Model does not contain 'X'");
   
   x = ukr_mx_get_matrix(prhs[0], model.q, 1);
   if (x==NULL) mexErrMsgTxt("Parameter x must be a 'q x 1' vector.");
   
   gs.y=gs.KyY=NULL;
   
   if (model.Y==NULL) {
      const double *pd;
      pd = ukr_mx_get_matrix(mxGetField(prhs[2],0,"d"),1,1);
      if (pd==NULL) mexErrMsgTxt(FS_ERR_MSG);
      gs.Kyy = *pd;
   
      gs.KyY = ukr_mx_get_matrix(mxGetField(prhs[2],0,"D"),model.N,1);
      if (gs.KyY==NULL) mexErrMsgTxt(FS_ERR_MSG);
   } else {
      gs.y = ukr_mx_get_matrix(prhs[2],model.d,1);
      if (gs.y==NULL) mexErrMsgTxt(DS_ERR_MSG);
   }
   
   if (0==ukr_aux_alloc_workspace(model,&W)) {
      mexErrMsgTxt("Sorry, out of memory.");
   }

	plhs[1]=mxCreateDoubleMatrix(model.q,1,mxREAL);
	g=mxGetPr(plhs[1]);
	plhs[2]=mxCreateDoubleMatrix(model.q,model.q,mxREAL);
	H=mxGetPr(plhs[2]);
   
   err = ukr_g_errgradhess(g,H,x,model,&W,gs,0);
   
   ukr_aux_free_workspace(W);
	plhs[0]=mxCreateDoubleScalar(err);
}
