#include <mex.h>
#include <matrix.h>
#include "ukr_matlab.h"
#include <string.h>

int calcNumNonZero(const double *D,int N) {
   int num = 0;
   int i;
   for (i=0;i<N;i++) 
      if (D[i]<1) num++;
   return num;
}

void mexFunction(int nlhs, mxArray *plhs[], int nrhs,const mxArray *prhs[]) {
const double *D;
const int *MaskI;
const int *MaskJ;
int MaskNZ;
int M,N;
int kernel;
const mxArray *ar;
char str[20];

   if (nrhs<2) mexErrMsgTxt("Not enough arguments.");
   if (nrhs>3) mexErrMsgTxt("Too many arguments.");
      
   D=mxGetPr(prhs[0]);
   M=mxGetM(prhs[0]);
   N=mxGetN(prhs[0]);
      
   ar=mxGetField(prhs[1],0,"kernel_l");
   if (ar==NULL) mexErrMsgTxt("No latent kernel given.");
   mxGetString(ar,str,20);
   kernel = ukr_kernel_by_name(str);
   if (kernel==UKR_KERNEL_UNKNOWN) mexErrMsgTxt("Unknown latent kernel.");
   
   if (nrhs==3 && !mxIsEmpty(prhs[2])) {
      if (!mxIsSparse(prhs[2]) || mxGetM(prhs[2])!=M || mxGetN(prhs[2])!=N) {
         mexErrMsgTxt("4th argument must be a sparse matrix with dimensions equal to the matrix of distances.");
      }
      MaskI=mxGetIr(prhs[2]);
      MaskJ=mxGetJc(prhs[2]);
   } else {
      MaskI=NULL;
      MaskJ=NULL;
   }
   
   if (kernel>UKR_KERNEL_SPARSE) {
      double *k,*p,*sumK;
      int j,*ir,*jc;
      int NNZ;
      Ukr_kp_dists_sparse kp_dists = ukr_get_kernel_sparse(kernel);
      
      NNZ = calcNumNonZero(D,M*N);
            
  		plhs[0]=mxCreateSparse(M,N,NNZ,mxREAL);
  		k=mxGetPr(plhs[0]);
		ir=mxGetIr(plhs[0]);
		jc=mxGetJc(plhs[0]);
		jc[0]=0;

      if (nlhs>1) {
         plhs[1]=mxCreateDoubleMatrix(1,N,mxREAL);
         sumK=mxGetPr(plhs[1]);
         if (nlhs>2) {
            plhs[2]=mxCreateSparse(M,N,NNZ,mxREAL);
            p=mxGetPr(plhs[2]);
         } else {
            p=NULL;
         }
      } else {
         sumK=NULL;
         p=NULL;
      }
      
      NNZ=0;
      
      for (j=0;j<N;j++) {
         int nz;
         double sj;
         
         if (MaskJ) {
            sj = kp_dists(M,D+j*M,k,p,ir,&nz,MaskJ[j+1]-MaskJ[j],MaskI+MaskJ[j]);
         } else {
            sj = kp_dists(M,D+j*M,k,p,ir,&nz,0,NULL);
         }
         if (sumK) sumK[j]=sj;
         k+=nz;
         if (p!=NULL) p+=nz;
         ir+=nz;
         NNZ+=nz;
         jc[j+1]=NNZ;
      }
      
      if (p) {
         memcpy(mxGetIr(plhs[2]),mxGetIr(plhs[0]),NNZ*sizeof(int));
         memcpy(mxGetJc(plhs[2]),mxGetJc(plhs[0]),(N+1)*sizeof(int));
      }
      
   } else {
      double *k,*p,*sumK;
      int j;
      int NNZ;
      Ukr_kp_dists_full kp_dists = ukr_get_kernel_full(kernel);
      
      NNZ = calcNumNonZero(D,M*N);

   
      plhs[0]=mxCreateDoubleMatrix(M,N,mxREAL);
            
  		k=mxGetPr(plhs[0]);

      if (nlhs>1) {
         plhs[1]=mxCreateDoubleMatrix(1,N,mxREAL);
         sumK=mxGetPr(plhs[1]);
         if (nlhs>2) {
            plhs[2]=mxCreateDoubleMatrix(M,N,mxREAL);
            p=mxGetPr(plhs[2]);
         } else {
            p=NULL;
         }
      } else {
         sumK=NULL;
         p=NULL;
      }
            
      for (j=0;j<N;j++) {
         double sj;
         
         if (MaskJ) {
            sj = kp_dists(M,D+j*M,k,p,MaskJ[j+1]-MaskJ[j],MaskI+MaskJ[j]);
         } else {
            sj = kp_dists(M,D+j*M,k,p,0,NULL);
         }
         if (sumK) sumK[j]=sj;
         k+=M;
         if (p) p+=M;
      }
   }  
}
