function D = ukr_aux_distances(A,B)
% function D = ukr_aux_distances(A,B)
% 
% calculates 2-norm distances between each columns of A and B.
%
% D(i,j) = norm(A(:,i)-B(:,j))
%
% D = distances(A) calculates distances(A,A)

if nargin<2
   [a N] = size(A);
   A2=sum(A.^2,1);
   D = repmat(A2',1,N) + repmat(A2,N,1) - 2*A'*A;
else 
   [a M] = size(A);
   [b N] = size(B);

   if a~=b
      error('Dimensions do not match');
   end

   A2 = sum(A.^2,1);
   B2 = sum(B.^2,1);

   D = repmat(A2',1,N) + repmat(B2,M,1) - 2*A'*B;
end
