function [err,GX,GXt] = ukr_aux_errgrad(X, Xt, T, model, Mask)
% [err,grad] = ukr_aux_c_errgrad(X, Xt, T, model, Mask)
%
% Normally you don't call this function directly.

[q,N]=size(X);
Nt = size(Xt,2);

if ~isfield(model,'degree') | model.degree==0

   [K,sumK,P] = ukr_kp_dists(ukr_aux_distances(X,Xt),model,Mask);
   iSumK=sumK.^(-1);
   iSumK(sumK==0)=0;
   B = K.*repmat(iSumK,N,1);
   P = P.*repmat(iSumK,N,1);

   if nargout>1
      [err,M] = ukr_aux_err_m(B,model,T);
      
      sBM = sum(B.*M,1);
      
%      R = P.*(M - repmat(sBM,N,1));
      R = P.*(M - ones(N,1)*sBM);

%      GX = (1/Nt)*(Xt*R' - X.*repmat(sum(R,2)',q,1));
%      GXt = (1/Nt)*(X*R - Xt.*repmat(sum(R,1),q,1));
      GX = (1/Nt)*(Xt*R' - X.*(ones(q,1)*sum(R,2)'));
      GXt = (1/Nt)*(X*R - Xt.*(ones(q,1)*sum(R,1)));
      
   else
      err = ukr_aux_err_m(B,model,T);
   end

   err = err/Nt;
else
   switch(model.degree)
      case 1
      case 2
      case 3
         if q~=2
            error 'Full quadratic model (degree=3) only implemented for q=2';
         end
      otherwise 
         error 'model.degree must be between 0 and 3 (q=2)';
   end

   err = 0;
   GX = zeros(q,N);
   GXt = zeros(q,Nt);

   for n=1:Nt
      Xn = X - repmat(Xt(:,n),1,N);

      [errn,XQ_T] = ukr_aux_XQ_T(Xn, model, Mask(:,n),T(:,n));
      err = err+errn;

      GX = GX + XQ_T;
      GXt(:,n) = GXt(:,n) - sum(XQ_T,2);
   end

   GX  = (-1/Nt)*GX;
   GXt = (-1/Nt)*GXt;
   err = err/Nt;
end
