function [B,dens] = ukr_b(X, model)
%UKR_b Compute basis function responses for one or several latent space elements.
%[B,dens] = ukr_b(X, model)
%
% Inputs:
%   X       -  Matrix of latent space elements (column-wise).
%   model   -  Valid ukr_model structure.
%
% Output:
%   B    -  Matrix of (normalized) basis function responses (column-wise).
%   dens -  row vector of densities at latent positions X

error(nargchk(2, 2, nargin))

[q N] = size(model.X);
[qT NT] = size(X);

if q~=qT; error('Dimensionalities do not match!'); end;

[K,sumK] = ukr_kp_dists(ukr_aux_distances(model.X,X),model);

if nargout>1
   dens = sumK/N;
end

if ~isfield(model,'degree') | model.degree==0
   B = ukr_aux_normalize(K,sumK);
else
   B = ukr_aux_lq_b(X,model,K);
end
