function ukr_buildmex(prefix)
% ukr_buildmex([prefix])
%
% This script builds "mex"-replacements for some
% ukr_toolbox functions. Make sure you have a
% working C compiler installed (try "mex -setup")
%
% Calling this script with an argument (e.g. 'T') will
% prefix all function names with that argument, e.g.
% build Tukr_errgrad.*** instead of ukr_errgrad.***

olddir=pwd;

% change to the 'mex' directory of the UKR toolbox
destdir=which('ukr_buildmex');
pos=strfind(destdir,'ukr_buildmex.m');
cd([destdir(1:pos-1) 'mex']);

fprintf(1,'Building C library ...\n');

mex -c -v specialmath.c
mex -c ukr.c
mex -c ukr_matlab.c

if ispc
   objs = 'mex/ukr_matlab.obj mex/ukr.obj mex/specialmath.obj';
else
   objs = 'mex/ukr_matlab.o mex/ukr.o mex/specialmath.o';
end
cd ..

funcs = {'ukr_f','ukr_fJ','ukr_kp_dists','ukr_b','ukr_bJ','ukr_errgrad',...
         'ukr_errgrad_scale','ukr_g_congrad','ukr_g_errgradhess',...
         'ukr_g_opt_unc','ukr_g_opt_boxcon','ukr_g_opt_denscon',...
         'ukr_aux_distances','ukr_aux_nearest',...
         'ukr_dens_penalty','ukr_densvar_penalty'};

if nargin==0
   for i=1:length(funcs)
      fprintf(1,'Building function "%s"...\n',funcs{i});
      mex(['mex/' funcs{i} '.c'],objs);
   end  
else
   for i=1:length(funcs)
      fprintf(1,'Building function "%s" as "%s"...\n',funcs{i},[prefix funcs{i}]);
      mex('-o',[prefix funcs{i}],['mex/' funcs{i} '.c'],objs);
   end  
end

cd(olddir);
