function [pen, grad, mindens] = ukr_dens_penalty(X, model,varargin)
%function [pen, grad, mindens] = ukr_dens_penalty(X, model [,eta])

error(nargchk(2, 3, nargin))

if nargin==3
   eta=varargin{1};
elseif isfield(model,'eta')
   eta=model.eta;
else
   eta=0;
end

% get dimensions:
[q N] = size(X);

% compute distance and kernel matrices
[K,sumK,P] = ukr_kp_dists(ukr_aux_distances(X),model);

P = ukr_aux_normalize(P,sumK-N*eta);

if nargout > 1
   R=-(P+P');
   r=sum(R);
   grad=X*R - X.*repmat(r,q,1);
end

dens=sumK/N;

pen=-sum(log(dens-eta));

if nargout>2
   mindens = min(dens);
end

