function [err, grad] = ukr_errgrad(X, model, cv)
% [err, grad] = ukr_errgrad(X,model [,cv]) 
%
% Compute mean squared reconstruction error and its gradient
%
% The flag "cv" determines if cross-validation is used
% If not given, model.Mask determines the kind of cross-validation
%   cv=0 : do not use cross-validation (overrides model.Mask)
%   cv~=0: use leave-one-out cross-validation (overrides model.Mask)
%
% Normally you don't call this function directly.

error(nargchk(2, 3, nargin))

[q,Na]=size(X);

if isfield(model,'G')
   N=size(model.G,2);
   T=1:N; % this is an implementation trick
   Nt=N;      
   if N~=Na
      error 'Dimensions of X and model.G do not match';
   end
   plain=1;
else
   if isfield(model,'Yt')
      T = model.Yt;
      [d,Nt]=size(T);
      [d2,N]=size(model.Y);
      if d~=d2
         error 'Dimension of model.Y and model.Yt do not match.';
      end
      if Na~=N+Nt
         error 'Dimensions of X=(Xs,Xt) does not match (model.Y,model.Yt)';        
      end
      plain=0;
   else
      T=model.Y;
      N=size(T,2);
      Nt=N;
      if N~=Na
         error 'Dimensions of X=(Xs,Xt) does not match model.Y';
      end
      plain=1;
   end
end

if nargin>2
   if cv
      if N~=Nt
         error 'LOO-CV cannot be applied in this case.'
      end
      Mask=speye(N);
   else
      Mask = sparse([],[],[],N,Nt,0);
   end
else
   if isfield(model,'Mask')
      if ~isequal(size(model.Mask),[N Nt])
         error 'Given CV-Mask does not match given latent variable matrix';
      end
      Mask = model.Mask;
   else
      Mask = sparse([],[],[],N,Nt,0);
   end
end

if plain 
   [err,GX,GXt] = ukr_aux_errgrad(X,X,T,model,Mask);
   grad = GX+GXt;
else
   [err,GX,GXt] = ukr_aux_errgrad(X(:,1:N),X(:,(N+1):end),T,model,Mask);
   grad = [GX GXt];
end
