function [err,gf,Hf] = ukr_g_errgradhess(x,model,y)
% function [err,gf,Hf] = ukr_g_errgradhess(x,model,y)
%
% Calculates the error, the gradient and an approximate Hessian
% for projecting a new data vector onto the UKR manifold.
% Normally, you won't use this directly

q=size(x,1);

if ~isfield(model,'G')
	[f,J]=ukr_fJ(x,model);
   
   f = f-y;
   
   if isfield(model,'mmm')
      % mmm stands for "multi map metric"
      %
      % Project onto UKR manifold by minimizing
      % |diag(mmm)*(f(x) - y)|^2
      %
      % Thus, mmm must be a column vector matching the 
      % data space dimensionality
      %
      f = f.*model.mmm;
      J = J.*repmat(model.mmm,1,q);
   end

	err = 0.5*sum(f.^2);

	gf = J'*f;
	Hf = J'*J;
else
	[b,Jb]=ukr_bJ(x,model);
		
	m = model.G*b-2*y.D;
	err = 0.5*(b'*m + y.d);
	
	JbG=Jb'*model.G;
	
	gf = 0.5*(Jb'*m + JbG*b);
	Hf = JbG*Jb;
end
