function x = ukr_g_opt_boxcon(y,model,x0)
% function x = ukr_g_opt_boxcon(y,model,x0)
%
% Bound constraint non-linear least squares
% used for projecting data vectors onto UKR manifolds in ukr_g
%
% Normally, you won't use this function directly

KKT_SENS=0.99;
CON_SENS=0.00001;
GRAD_TOL=1e-8;
MAX_ITER=50;

lb = model.lb;
ub = model.ub;

indu = find(x0>=ub);
indl = find(x0<=lb);

x=x0;

x(indu)=ub(indu);
x(indl)=lb(indl);

[f,gf,Hf]=ukr_g_errgradhess(x,model,y);

iter = 0;
q = length(x);
lambda=0.1*f;

if isfield(model,'display') & model.display
	disp=1;
else
	disp=0;
end

while iter<MAX_ITER
	indl = indl(gf(indl)>0);
	indu = indu(gf(indu)<0);
	
	ind = setdiff(1:q,[indl ;indu]);
		
%	fprintf(1,'%i ',[indl; indu]);
%	fprintf(1,'%f ',x)
	fprintf(1,'[%3i] %f\t%f\n',iter,f,lambda);	

	if norm(gf(ind)) < GRAD_TOL 
%		fprintf(1,'KKT point\n');
		break;
	end
		
	H = Hf(ind,ind) + lambda*eye(length(ind));
	
	dx = -H\gf(ind);
				
	pred=0.5*dx'*H*dx + gf(ind)'*dx;
	
	oldx = x;
	x(ind) = x(ind) + dx;
	
	indu = find(x>=ub);
	indl = find(x<=lb);
	x(indu)=ub(indu);
	x(indl)=lb(indl);
	
	iter=iter+1;
	
	oldf = f;
   [f,gf,Hf]=ukr_g_errgradhess(x,model,y);   
	
	if f>oldf+0.5*pred
		lambda=lambda*5;
	else
		lambda=lambda*0.9;
	end
end
		
	
