function x = ukr_g_opt(y,model,x0)
% function x = ukr_g_opt(y,model,x0)
%
% Nonlinear least squares optimization 
% used for projecting data vectors onto UKR manifolds in ukr_g
%
% Normally, you won't use this function directly

KKT_SENS=0.99;
CON_SENS=0.00001;
GRAD_SENS=1e-8;
MAX_ITER=50;
MIN_STEP=0.000001;

x=x0;

[f,gf,Hf]=ukr_g_errgradhess(x,model,y);
ngf=norm(gf);

iter = 0;
q = length(x);
lambda=0.1*f;

if isfield(model,'display') & model.display
	disp=1;
else
	disp=0;
end


while iter<MAX_ITER

	if disp
		fprintf(1,'[%3i] %f\t%f\t%f\n',iter,f,lambda,x(1));	
	end
		
   if ngf < GRAD_SENS
%     fprint(1,'Mimimum found');
      break;
   end
	
	H = Hf + lambda*eye(q);
   dx = -H\gf;
   
   pred=0.5*gf'*dx;
	
	x = x + dx;
	
	if norm(dx)<MIN_STEP break; end
	
	iter=iter+1;
	
	oldf = f;
	
	[f,gf,Hf]=ukr_g_errgradhess(x,model,y);
   ngf=norm(gf);   
	
	if f>oldf+0.5*pred
		lambda=lambda*5;
	else
		lambda=lambda*0.9;
	end
end
		
