function [pen, grad, mindens] = ukr_ldens_penalty(XXt, model,varargin)
%function [pen, grad, mindens] = ukr_ldens_penalty(XXt, model [,eta])

error(nargchk(2, 3, nargin))

if nargin==3
   eta=varargin{1};
elseif isfield(model,'eta')
   eta=model.eta;
else
   eta=0;
end

% get dimensions:
[q NNt] = size(XXt);

N = size(model.Y,2);
Nt = size(model.Yt,2);

if N+Nt~=NNt
   error 'Dimension of 1. parameter (XXt) does not match model.';
end

X = XXt(:,1:N);
Xt = XXt(:,(N+1):NNt);

% compute distance and kernel matrices
[K,sumK,P] = ukr_kp_dists(ukr_aux_distances(X,Xt),model);

P = ukr_aux_normalize(P,sumK-N*eta);

if nargout > 1
   R=-P;
   r=sum(R);
   r2=sum(R,2)';
   grad1=X*R - Xt.*repmat(r,q,1);
   grad2=Xt*R' - X.*repmat(r2,q,1);
   grad = [grad2 grad1];
end

dens=sumK/N;

pen=-sum(log(dens-eta));

if nargout>2
   mindens = min(dens);
end

