function [err,G]=ukr_loss_function(Diff,model)


[d,N]=size(Diff);

switch model.loss
   case 'L2'
      err = norm(Diff,'fro')^2;
      if nargout>1
         G = 2*Diff;
      end
   case 'Eps'
      res = abs(Diff);
      if equals(size(model.eps),[d 1])
         loss = res - repmat(model.eps,1,N);
      else
         loss = res - model.eps;
      end
      loss(loss<0)=0;
      err = sum(sum(loss.^2));
      if nargout>1
         G = 2*sign(Diff).*loss;
      end
   case 'Eps2'
      res = sqrt(sum(Diff.^2));
      loss = (res - model.eps);
      loss(loss<0)=0;
      err = sum(loss.^2);

      res(res==0)=1;
      if nargout>1
         G = Diff.*repmat(2*loss./(res),d,1);
      end
   case 'Huber'
      res = abs(Diff);
      
      if length(model.eps)==1
         indS = find(res<model.eps);
         indL = find(res>=model.eps);

         res(indS) = res(indS).^2/(2*model.eps);
         res(indL) = res(indL) - model.eps*0.5;

         err = sum(sum(res));

         if nargout>1
   	      G = Diff/model.eps;
   	      G(indL)=sign(Diff(indL));
         end
      else
         if equals(size(model.eps),[d 1])
            EPS = repmat(model.eps,1,N);
         else
            error 'Huber loss: Dimensions of model.eps do not match model';
         end
         indS = find(res<EPS);
         indL = find(res>=EPS);

         res(indS) = res(indS).^2./(2*EPS);
         res(indL) = res(indL) - 0.5*EPS;

         err = sum(sum(res));

         if nargout>1
   	      G = Diff./model.eps;
   	      G(indL)=sign(Diff(indL));
         end
      end
   otherwise
      if exist(model.loss)==2
         [err,G]=feval(model.loss,Diff,model);
      else
         error 'Unsupported loss function';
      end
end
         
