function [Yout, index] = ukr_nr(Yin, model)
% UKR_NR Determine 'nearest reconstruction' for observable space elements.
% [Yout, index] = ukr_nr(Yin, model)
%
% Inputs:
%   Yin    -  Observable data space elements (in columns)
%   model  -  Valid ukr_model structure.
%
% Outputs:
%   Yout   -  Among the model's observable data 'reconstructions' (i.e. the images
%             under ukr_f of all input model's latent space elements) the ones with the
%             smallest Euclidean distance to Yin (columnwise).
%   index  -  The corresponding latent space elements indices with regard to the
%             model's latent data matrix.
%
% Notice : You should not call this function if using Kernel UKR.
% Use ukr_nfr instead.

error(nargchk(2, 2, nargin))

% get dimensions:
[d N] = size(model.Y);

if isfield(model,'Yr')
   if isfield(model,'mmm')
      M = diag(model.mmm);
      index = ukr_aux_nearest(M*Yin, M*model.Yr);
   else
      index = ukr_aux_nearest(Yin,model.Yr);
   end
   Yout = model.Yr(:,index);
else
   Yr = ukr_f(model.X,model);
   index = ukr_aux_nearest(Yin,Yr);
   Yout = Yr(:,index);
end


