function [X,f] = ukr_opt_denscon(Xinit, maxiter, model, eta_con, varargin)
% function [X,f] = ukr_opt_denscon(Xinit, maxiter, model, eta_con, varargin)
%
% Density constraint optimization of R(X), uses a barrier function
% to keep all densities p(x_i) > eta_con

densTol = 0.0001;
MAXBACK=8;
X=Xinit;
gradBound2=length(X)*(10^-16);

rs = ukr_rprop_init(X);

lambda=0.00001;

[pen,pg,md] = ukr_dens_penalty(X,model,eta_con);

while md<=eta_con
   fprintf(1,'Invalid initialization - Scaling down...\n');
   X=X*0.9;
   [pen,pg,md] = ukr_dens_penalty(X,model,eta_con);
end
  
[f,g]=ukr_errgrad(X,model,varargin{:});

disp=0;
if isfield(model,'display')
   disp=model.display;
end

for iter=0:maxiter

   abs2g=sum(sum(g.^2));
   abs2pg=sum(sum(pg.^2));

   if abs2g<=gradBound2
      return
   end
   
   if (md - eta_con) < densTol
      if sum(sum(g.*pg))>0.97*sqrt(abs2g*abs2pg)  % KKT
         fprintf(1,'KKT condition');
         return
      end
   end
   
   [X, rs]=ukr_rprop_adapt(X, rs, g + lambda*pg);
   oldF=f;

   [pen,pg,md] = ukr_dens_penalty(X,model,eta_con);
   
   counter = 0;
   
   while md<=eta_con & counter<MAXBACK
      rs.eta=rs.eta*0.9;
      counter=counter+1;
      X=0.5*(X+rs.oldX);
      [pen,pg,md] = ukr_dens_penalty(X,model,eta_con);         
   end
   
   if md<=eta_con
      X=rs.oldX;
      f=oldF;
      break;
   elseif counter==0
      lambda=lambda*0.8;
   else
      lambda=lambda*1.5;
   end
   
   [f,g]=ukr_errgrad(X,model,varargin{:});

   if disp
      fprintf(1,'[%3i] %f  p_min=%f ( %f )\n',iter,f,md,md-eta_con);
   end
end
