function X = ukr_opt_eps(objgrad,Xinit,maxiter,model)
% function X = ukr_opt_eps(objgrad,Xinit,maxiter,model)
%
% Specialized optimization routine to fit an epsilon-insensitive
% UKR model. Normally, you won't call this function directly.
% Constraint is realized by external penalty with increasing pre-factors.
%
% The series of pre-factors is hard-coded and may not be optimal
%
% objgrad   - Objective function and gradient, either
%              @ukr_ext_penalty or @ukr_dens_penalty
% Xinit     - initial latent variables (e.g. CV-regularized solution)
% maxiter   - maximum number of RPROP iterations carried out for each
%             of the 10 different values of the pre-factor lambda
% model     - UKR model

X=Xinit;
gradBound2=length(X)*(10^-16);

rs = ukr_rprop_init(X);

penTol = 1e-6;

disp=0;
if isfield(model,'display')
   disp=model.display;
end

lambdas = 10.^(4:2:16);  % 2:11

oIt = 0;

for lambda=lambdas
   oIt=oIt+1;
   [pen,pg] = ukr_errgrad(X,model);
   [f,g] = feval(objgrad,X,model);

   abs2g=sum(sum(g.^2));
   if pen>0
%      g=g+lambda*pen*pg;
      g=g+lambda*pg;      
   end

   found=0;
   if disp
      fprintf(1,'lambda=%f (Iteration %i/%i)\n',lambda,oIt,length(lambdas));
      fprintf(1,'[%3i] %f\n',0,f);
   end
   iter=1;

   while (~found)&(iter<=maxiter)
      [X, rs]=ukr_rprop_adapt(X, rs, g);
      oldF=f;

      [pen,pg] = ukr_errgrad(X,model);
      [f,g] = feval(objgrad,X,model);

      abs2g=sum(sum(g.^2));
      abs2pg=sum(sum(pg.^2));
      
      ac = sum(sum(g.*pg))/(sqrt(abs2g)*sqrt(abs2pg));

      if pen>0
%         g=g+lambda*pen*pg;
         g=g+lambda*pg;         
      end
      
      if disp
         fprintf(1,'[%3i] %f  %g\n',iter,f,pen);
      end

      found=(pen<=0 & abs2g<gradBound2) | (pen<penTol & ac>0.99);

      iter=iter+1;
   end
end
