function [Yout,Xout] = ukr_proj(Yin, model)
% Compute orthogonal projection onto a ukr manifold for one 
% or several observable space data elements.
%
% [Yout,Xout] = ukr_proj(Yin, model)
%
% Inputs:
%   Yin    -  Matrix of observable data set to be projected (columnwise).
%   model  -  Valid ukr_model structure.
%
% Output:
%   Yout  -  Matrix of projections (columnwise).
%   Xout  -  Result of applying the UKR-'backward mapping' (see ukr_g)
%
% Yields the element(s) on the UKR manifold (implicitly defined by 
% input ukr_model structure model) with the smallest Euclidean distance 
% to the corresponding input observable space element(s) in Yin.
% 
% See also ukr_g, ukr_f

error(nargchk(2, 2, nargin))

Xout = ukr_g(Yin, model);
Yout = ukr_f(Xout, model);

