function num = ukr_test_suite
% UKR toolbox test suite

%ukr_buildmex('T');

N = 30;

model.q = 2;
model.Y = rand(10,N);
model.X = rand(2,N);
s=0.5+rand(2,1);
x=0.25+0.5*rand(2,1);
y=rand(10,1);

KERNs = {'Gaussian', 'Quartic', 'Triweight'};
DEG = {'Constant', 'Linear', 'Quadratic w/o cross-terms', 'Full quadratic'};

num = 0;
thresh = 1e-4;

fprintf(1,'UKR models with a single set of data and latent variables.\n');
fprintf(1,'Checking errgrad, errgrad_scale and ukr_f against ukr_b.\n');
for k = 1:length(KERNs)
   model.kernel_l = KERNs{k};
   for deg = 0:3
      model.degree=deg;
      fprintf(1,'\n%s kernel, %s model\n',KERNs{k},DEG{deg+1});
      Yr = model.Y*ukr_b(model.X,model);
      errb = norm(model.Y - Yr,'fro')^2 / N;
      Yr = ukr_f(model.X,model);
      errf = norm(model.Y - Yr,'fro')^2 / N;      
      fprintf(1,'* ukr_f...............: %f\n',abs(errf - errb));
      errg = ukr_errgrad(model.X,model);
      fprintf(1,'* ukr_errgrad.........: %f\n',abs(errg - errb));
      errs = ukr_errgrad_scale([1;1],model,model.X);
      fprintf(1,'* ukr_errgrad_scale...: %f\n',abs(errs - errb));
   end
end

disp('Press any key'); pause;

fprintf(1,'Feature space UKR models (simple dot product).\n');
fprintf(1,'Checking errgrad and errgrad_scale against ukr_b.\n');
model.G = model.Y'*model.Y;
Ysafe = model.Y;
model = rmfield(model,'Y');

for k = 1:length(KERNs)
   model.kernel_l = KERNs{k};
   for deg = 0:3
      fprintf(1,'\n%s kernel, %s model\n',KERNs{k},DEG{deg+1});
      Yr = Ysafe*ukr_b(model.X,model);
      errb = norm(Ysafe - Yr,'fro')^2 / N;
%      Yr = ukr_f(model.X,model);
%      errf = norm(model.Y - Yr,'fro')^2 / N;      
%      fprintf(1,'* ukr_errgrad.........: %f\n',abs(errf - errb));
      errg = ukr_errgrad(model.X,model);
      fprintf(1,'* ukr_errgrad.........: %f\n',abs(errg - errb));
      errs = ukr_errgrad_scale([1;1],model,model.X);
      fprintf(1,'* ukr_errgrad_scale...: %f\n',abs(errs - errb));
   end
end

disp('Press any key'); pause;
model.Y = Ysafe;
model = rmfield(model,'G');

fprintf(1,'Comparing analytic derivatives with finite differences.\n');
for k = 1:length(KERNs)
   model.kernel_l = KERNs{k};

   fprintf(1,'\n%s kernel, independent of polynomial degree\n',KERNs{k});
   md = checkGrad(@ukr_dens_penalty,model.X,model);
   fprintf(1,'* ukr_dens_penalty....: %f\n',md);
   num = num + (md>thresh);            
   md = checkGrad(@ukr_densvar_penalty,model.X,model);
   fprintf(1,'* ukr_densvar_penalty.: %f\n',md);
   num = num + (md>thresh);            

   for deg = 0:3
      fprintf(1,'\n%s kernel, %s model\n',KERNs{k},DEG{deg+1});
      model.degree=deg;
      md = checkGrad(@ukr_errgrad,model.X,model,1);
      num = num + (md>thresh);
      fprintf(1,'* ukr_errgrad.........: %f\n',md);
      md = checkGrad(@ukr_errgrad_scale,s,model,model.X,1);
      num = num + (md>thresh);      
      fprintf(1,'* ukr_errgrad_scale...: %f\n',md);
      md = checkJacobi(@ukr_bJ,x,model);
      num = num + (md>thresh);      
      fprintf(1,'* ukr_bJ..............: %f\n',md);
      md = checkJacobi(@ukr_fJ,x,model);
      num = num + (md>thresh);      
      fprintf(1,'* ukr_fJ..............: %f\n',md);
      md = checkGrad(@ukr_g_errgradhess,x,model,y);
      fprintf(1,'* ukr_g_errgradhess...: %f\n',md);
      num = num + (md>thresh);      
   end
end

fprintf(1,'\nCount of too large deviations (> %f) : %i\n',thresh, num);



function md = checkGrad(fg,x,varargin)

N = length(x(:));

[f,g]=feval(fg,x,varargin{:});

dg = zeros(size(g));
h=1e-9;

for n=1:N
   xp=x;
   xp(n)=x(n)+h;
   fp=feval(fg,xp,varargin{:});
   dg(n) = (fp-f)/h;
end

md = max(max(abs(dg-g)))/abs(f);



function md = checkJacobi(fJ,x,varargin)

N = size(x,1);

[f,J]=feval(fJ,x,varargin{:});

dJ = zeros(size(J));
h=1e-7;

for n=1:N
   xp=x;
   xp(n)=x(n)+h;
   fp=feval(fJ,xp,varargin{:});
   dJ(:,n) = (fp-f)/h;
end

md = max(max(abs(dJ-J)))/norm(f,'fro');

